@@ -30,7 +30,7 @@ from ._artifactcache import ArtifactCache
 from ._elementsourcescache import ElementSourcesCache
 from ._remotespec import RemoteSpec, RemoteExecutionSpec
 from ._sourcecache import SourceCache
-from ._cas import CASCache, CASLogLevel
+from ._cas import CASCache, CASDProcessManager, CASLogLevel
 from .types import _CacheBuildTrees, _PipelineSelection, _SchedulerErrorAction, _SourceUriPolicy
 from ._workspaces import Workspaces, WorkspaceProjectCache
 from .node import Node, MappingNode
@@ -185,6 +185,12 @@ class Context:
         # User specified cache quota, used for display messages
         self.config_cache_quota_string: Optional[str] = None
 
+        # Reserved disk space for local cache in bytes
+        self.config_cache_reserved: Optional[int] = None
+
+        # Low watermark for local cache (ratio relative to effective quota)
+        self.config_cache_low_watermark: Optional[float] = None
+
         # Remote cache server
         self.remote_cache_spec: Optional[RemoteSpec] = None
 
@@ -223,6 +229,7 @@ class Context:
         self._project_overrides: MappingNode = Node.from_dict({})
         self._workspaces: Optional[Workspaces] = None
         self._workspace_project_cache: WorkspaceProjectCache = WorkspaceProjectCache()
+        self._casd: Optional[CASDProcessManager] = None
         self._cascache: Optional[CASCache] = None
 
     # __enter__()
@@ -247,7 +254,11 @@ class Context:
             self._sourcecache.release_resources()
 
         if self._cascache:
-            self._cascache.release_resources(self.messenger)
+            self._cascache.release_resources()
+
+        if self._casd:
+            self._casd.release_resources(self.messenger)
+            self._casd = None
 
     # load()
     #
@@ -357,7 +368,9 @@ class Context:
         # We need to find the first existing directory in the path of our
         # casdir - the casdir may not have been created yet.
         cache = defaults.get_mapping("cache")
-        cache.validate_keys(["quota", "storage-service", "pull-buildtrees", "cache-buildtrees"])
+        cache.validate_keys(
+            ["quota", "reserved-disk-space", "low-watermark", "storage-service", "pull-buildtrees", "cache-buildtrees"]
+        )
 
         cas_volume = self.casdir
         while not os.path.exists(cas_volume):
@@ -373,6 +386,32 @@ class Context:
                 LoadErrorReason.INVALID_DATA,
             ) from e
 
+        cache_reserved_string = cache.get_str("reserved-disk-space")
+        try:
+            self.config_cache_reserved = utils._parse_size(cache_reserved_string, cas_volume)
+            if self.config_cache_reserved is None:
+                provenance = cache.get_scalar("reserved-disk-space").get_provenance()
+                raise LoadError(
+                    "{}: Please specify the value in bytes or as a % of full disk space.\n"
+                    "\nValid values are, for example: 2G 5%\n".format(provenance),
+                    LoadErrorReason.INVALID_DATA,
+                )
+        except utils.UtilError as e:
+            raise LoadError(
+                "{}\nPlease specify the value in bytes or as a % of full disk space.\n"
+                "\nValid values are, for example: 2G 5%\n".format(str(e)),
+                LoadErrorReason.INVALID_DATA,
+            ) from e
+
+        low_watermark_string = cache.get_str("low-watermark")
+        try:
+            self.config_cache_low_watermark = utils._parse_percentage(low_watermark_string)
+        except utils.UtilError as e:
+            raise LoadError(
+                "{}\nPlease specify the value as a % of the cache quota.".format(str(e)),
+                LoadErrorReason.INVALID_DATA,
+            ) from e
+
         remote_cache = cache.get_mapping("storage-service", default=None)
         if remote_cache:
             self.remote_cache_spec = RemoteSpec.new_from_node(remote_cache)
@@ -677,8 +716,8 @@ class Context:
         # value which we cache here too.
         return self._strict_build_plan
 
-    def get_cascache(self) -> CASCache:
-        if self._cascache is None:
+    def get_casd(self) -> CASDProcessManager:
+        if self._casd is None:
             if self.log_debug:
                 log_level = CASLogLevel.TRACE
             elif self.log_verbose:
@@ -686,15 +725,24 @@ class Context:
             else:
                 log_level = CASLogLevel.WARNING
 
-            self._cascache = CASCache(
+            assert self.logdir is not None, "log_directory is required for casd"
+            log_dir = os.path.join(self.logdir, "_casd")
+            self._casd = CASDProcessManager(
                 self.cachedir,
-                casd=self.use_casd,
-                cache_quota=self.config_cache_quota,
-                remote_cache_spec=self.remote_cache_spec,
-                log_level=log_level,
-                log_directory=self.logdir,
+                log_dir,
+                log_level,
+                self.config_cache_quota,
+                self.remote_cache_spec,
+                protect_session_blobs=True,
                 messenger=self.messenger,
+                reserved=self.config_cache_reserved,
+                low_watermark=self.config_cache_low_watermark,
             )
+        return self._casd
+
+    def get_cascache(self) -> CASCache:
+        if self._cascache is None:
+            self._cascache = CASCache(self.cachedir, casd=self.get_casd(), remote_cache=bool(self.remote_cache_spec))
         return self._cascache
 
     ######################################################
