@@ -409,7 +409,7 @@ def help_command(ctx, command):
 @click.option(
     "--min-version",
     type=click.STRING,
-    default="2.4",
+    default="2.5",
     show_default=True,
     help="The required format version",
 )
@@ -613,6 +613,8 @@ def show(app, elements, deps, except_, order, format_):
         %{deps}           A list of all dependencies
         %{build-deps}     A list of build dependencies
         %{runtime-deps}   A list of runtime dependencies
+        %{source-info}    Source provenance information
+        %{artifact-cas-digest} The CAS digest of the built artifact
 
     The value of the %{symbol} without the leading '%' character is understood
     as a pythonic formatting string, so python formatting features apply,
@@ -638,7 +640,8 @@ def show(app, elements, deps, except_, order, format_):
         state_match = re.search(r"%(\{(state)[^%]*?\})", format_)
         key_match = re.search(r"%(\{(key)[^%]*?\})", format_)
         full_key_match = re.search(r"%(\{(full-key)[^%]*?\})", format_)
-        need_state = bool(state_match or key_match or full_key_match)
+        artifact_cas_digest_match = re.search(r"%(\{(artifact-cas-digest)[^%]*?\})", format_)
+        need_state = bool(state_match or key_match or full_key_match or artifact_cas_digest_match)
 
         if not elements:
             elements = app.project.get_default_targets()
