@@ -437,6 +437,40 @@ class LogLine(Widget):
                 runtime_deps = [e._get_full_name() for e in element._dependencies(_Scope.RUN, recurse=False)]
                 line = p.fmt_subst(line, "runtime-deps", _yaml.roundtrip_dump_string(runtime_deps).rstrip("\n"))
 
+            # Source Information
+            if "%{source-info" in format_:
+
+                # Get all the SourceInfo objects
+                #
+                all_source_infos = []
+                for source in element.sources():
+                    source_infos = source.collect_source_info()
+
+                    if source_infos is not None:
+                        serialized_sources = []
+                        for s in source_infos:
+                            serialized = s.serialize()
+                            serialized_sources.append(serialized)
+
+                        all_source_infos += serialized_sources
+
+                # Dump the SourceInfo provenance objects in yaml format
+                line = p.fmt_subst(line, "source-info", _yaml.roundtrip_dump_string(all_source_infos))
+
+            # Artifact CAS Digest
+            if "%{artifact-cas-digest" in format_:
+                artifact = element._get_artifact()
+                if artifact.cached():
+                    artifact_files = artifact.get_files()
+                    # We call the private CasBasedDirectory._get_digest() for
+                    # the moment, we should make it public on Directory.
+                    artifact_digest = artifact_files._get_digest()
+                    formated_artifact_digest = "{}/{}".format(artifact_digest.hash, artifact_digest.size_bytes)
+                    line = p.fmt_subst(line, "artifact-cas-digest", formated_artifact_digest)
+                else:
+                    line = p.fmt_subst(line, "artifact-cas-digest", "")
+                    element.warn("Cannot obtain CAS digest because artifact is not cached")
+
             report += line + "\n"
 
         return report.rstrip("\n")
